
/*******************************************************************************

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Copyright (C) 2011 Nuno Fortes(nunofort@gmail.com)

FILE: Agent.hpp

DESC: Agent main program class

********************************************************************************/

#include <pcre++.h>
#include <c++/version/string>
#include <c++/version/sstream>
#include <c++/version/iostream>
#include <c++/version/fstream>

using namespace std;
using namespace pcrepp;

#include <time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/socket.h>
#include <regex.h>

//extern "C++" {
//#include "Monitor.hpp"
#include "LogMonitor.hpp"
#include "NetMonitor.hpp"
//#include "Parser.hpp"
//#include "Guard.hpp"
//}


class Agent
{
public:
Agent();
void print_config();
void print_only();
void test_msg(const char *msg);
int start();
void about(const char *bin);
void initialize(const char *conf);
void setVerbose(bool verbose);
int getMode();

private:

int is_numeric(const char *text);
void error(std::string msg, std::string filename);
//remove?
void load_config(const char *conf);

     int mode;
     bool verbose;

     std::string errfile;
     std::string logfile;
     std::string posfile;
     std::string ipsfile;
     std::string cntfile;
     int interval; // seconds
     int port;

     Monitor *monitor;
     Parser parser;
     Guard guard;

     //vector<std::string> regex_vars; // Variables to check values for the regex
     //vector<std::string> regex_varnames; // the names of the variables ${0} ?
     //vector<std::string> keys; // 256*32 = 7K ? Mapping keys
     // directly map to $0 $1 from regex not matched ?

     //vector<std::string> regex; // C Regex functions
     //vector<std::string> exec;

     int LOG_LOCK;
};



Agent::Agent() 
: mode(1), port(0),verbose(false) { 
  LOG_LOCK = 0;

  //guard = Guard;

  //initialize(config);
}

void Agent::setVerbose(bool verbose) {
this->verbose = verbose;
}

void Agent::error(std::string msg,std::string filename)
{
    //std::ofstream ofs;
    //ofs.open(filename,ios::binary|ios::app);
    cout << msg << endl;
    exit(0);	
}

int Agent::is_numeric(const char *text)
{
  int i, ii;
  char DECNUM[] = {'0','1','2','3','4','5','6','7','8','9'};
  int found = 0;
  for (i = 0; text[i] != '\0'; i++)
  {
        for (ii = 0; ii < sizeof(DECNUM); ii++)
        if (text[i] == DECNUM[ii]) found++;
  }
  return (found == strlen(text));
}

void Agent::load_config(const char *conf)
{
  FILE *in;
  char key[512];
  char *value;
  in = fopen(conf,"r");
  do
  {
        //TODO: change parser to OO, only functions for the app!
        value = parser.read_config(in,key,'=');
	if ((key[0] != '#') && (value != NULL)) 
	{
           if (verbose) printf("LOAD %s=[%s]\n",key,value);
	   if (strcmp(key,"mode") == 0) {
                if (strcmp(value,"normal") == 0)
                   mode = 1;
                else if (strcmp(value,"daemon") == 0)
                   mode = 2;
                else
                   mode = 1;
	   } else if (strcmp(key,"log") == 0) {
		this->logfile = value;
	   } else if (strcmp(key,"err") == 0) {
		errfile = value;	
	   }
	   else if (strcmp(key,"pos") == 0)
	   {
                posfile = value;
		//strcpy(posfile,value);	
	   }
	   else if (strcmp(key,"ips") == 0)
	   {
		ipsfile = value;	
	   }
	   else if (strcmp(key,"cnt") == 0)
	   {
		cntfile = value;	
	   }
	   else if (strcmp(key,"port") == 0)
	   {
		port = atoi(value);	
	   }
/*
	   else if (strcmp(key,"ip") == 0)
	   {
		strcpy(ip,value);	
	   }
*/
	   else if (strcmp(key,"interval") == 0)
	   {
		interval = atoi(value);	
	   }
	   else if (strcmp(key,"regex") == 0)
	   {
		guard.addExpression(value);
	   }
           else if (key[0] == '%') {
                std::string skey(key);
                std::string s(skey,1);
                //cout << "IDX:" << key << " : " << s << endl;
                if (is_numeric(s.c_str()) == 1) {
                // ADD MATCH INDEX NUMBER
                guard.addMatch(atoi(s.c_str()));
                //guard.idxMatch(s.toInt());
                //cout << "SUBREGEX:" << value << endl;
                //ADD SUB REG EXPRESSION
                guard.addExpression(value);
                }
           }
/*
	   else if (strcmp(key,"regexvar") == 0)
	   {
		//strcat
		regexvar.push_back(value);
	   }
	   //else if (is_numeric(key) == 1)
	   else if (strcmp(key,"action") == 0)
	   {
		guard.addMapping(atoi(value));
	   }
*/
	   else if (strcmp(key,"action") == 0)
	   {
		guard.addAction(0,value);
	   }

	}
  }
  while (!parser.real_end(in));
  fclose(in);
}

int Agent::getMode() {
   return mode;
}

void Agent::print_config()
{
  int i;
  cout << "ERR FILE: "<< errfile << endl;
  cout << "LOG FILE: "<< logfile << endl;
  cout << "POS FILE: "<< posfile << endl;
  //printf("IPS FILE: %s\n",ipsfile);
  //printf("    PORT: %u\n",port);
  //printf("      IP: %u\n",ip);
  cout << "INTERVAL: " << interval << endl;
  cout << guard << endl;

}

void Agent::print_only()
{ 
  printf("[+] Print Config Options...\n"); 
  this->print_config();
}


void Agent::initialize(const char *conf)
{ 
  cout << "Initializing Guard..." << endl;
  cout << "[+] Loading Config..." << endl;
  load_config(conf);

  //printf("[+] Print Config Options...\n"); 
  //print_config();
  //exit(0);
  // check regular expressions integrity
  //guard.validate_rules();
}

void Agent::test_msg(const char *msg)
{
  printf("[+] Checking Message...\n"); 
  //monitor = new TestMonitor(msg);
  //monitor->process(guard);
  // WARN: GET BIG LINES
  guard.checkMessage(msg);
}

int Agent::start()
{
  if (port == 0) { 
  //printf("[+] Log File Monitoring...\n"); 
  monitor = new LogMonitor(logfile,posfile);
  } else {
  //printf("[+] Network Monitoring...\n"); 
  monitor = new NetMonitor(ipsfile,port);
  }
  guard.setVerbose(this->verbose);
  monitor->process(guard);
  printf("[+] DONE!\n"); 
  //else
	//this.net_monitor();
  return interval;
} 


void Agent::about(const char *bin)
{
   printf("Guardog %s    Intrusion Dectection System (IDS)\n",__PROGRAM_VERSION);
   printf("                by Nuno Fortes (nunofort@gmail.com)\n\n");
   printf("syntax: %s -c [config_file]\n\n",bin);
   printf("syntax: %s -p [config_file] (print only info)\n\n",bin);
   printf("syntax: %s -t [config_file] [test_msg] (test msg against regex config)\n\n",bin);
}



